var BlockSite = 
{
	checkArrayR: function(arr, value)
	{
		for (var i=0; i < arr.length; i++) 
		{
			var regexp = convert2RegExp(arr[i]);
			if(regexp)
			{
				if(regexp.test(value))
				{
					return true;
				}
			}
		}
    
		return false;
	},

	showPrefs: function(notification, description)
	{
		window.open("chrome://blocksite/content/BlockSitePrefs.xul", "BlockSite Preferences", "chrome,centerscreen,resizable=no");
	},	
	
	showBlockWarningBar: function()
	{
		var messageBarText = document.getElementById("blocksite.locale").getString("BlockSite.messageBarText");
                
        var notificationBox = gBrowser.getNotificationBox();
		var notification = notificationBox.getNotificationWithValue("website-blocked");
		
		if(notification)
		{
			notification.label = messageBarText;
		}
		else
		{
			const priority = notificationBox.PRIORITY_WARNING_MEDIUM;
			notificationBox.appendNotification(messageBarText, "website-blocked", "chrome://browser/skin/Info.png", priority, null);
		}
	},
	
	isBlackList: function(prefBranch)
	{
		if (!prefBranch.prefHasUserValue("listtype"))
			return true;
		return prefBranch.getCharPref("listtype") == "blacklistRadio";
	},
	
	checkLocation: function(location)
	{
		const BlockSitePrefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("BlockSite.");
		
		if(BlockSitePrefBranch.prefHasUserValue("locations"))
		{
			var blockedLocationsString = BlockSitePrefBranch.getComplexValue("locations", Components.interfaces.nsISupportsString).data;
			if(blockedLocationsString != "")
			{
				var blockedLocationsArray = blockedLocationsString.split("|||");
				
				if(this.checkArrayR(blockedLocationsArray, location))
					return this.isBlackList(BlockSitePrefBranch);
			}
		}
		
		return !this.isBlackList(BlockSitePrefBranch);
	},
	
	checkAnchor: function(anchor)
	{
		const BlockSitePrefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("BlockSite.");
		
		if(BlockSitePrefBranch.prefHasUserValue("locations"))
		{
			var blockedLocationsString = BlockSitePrefBranch.getComplexValue("locations", Components.interfaces.nsISupportsString).data;
			if(blockedLocationsString != "")
			{
				var blockedLocationsArray = blockedLocationsString.split("|||");
				
				if(this.checkArrayR(blockedLocationsArray, anchor.href))
					return this.isBlackList(BlockSitePrefBranch);
			}
		}
		
		return !this.isBlackList(BlockSitePrefBranch);
	},
	
	processAnchors: function(event)
	{
		var anchorElements = event.target.getElementsByTagName("a");
			
		for(var i=0; i < anchorElements.length; i++)
		{
			if(BlockSite.checkAnchor(anchorElements[i]))
			{
				var tempFragment = document.createDocumentFragment();
				var childNodes = anchorElements[i].childNodes;
				var parentNode = anchorElements[i].parentNode;
				
				for(var j=0; j < childNodes.length; j++)
				{
					tempFragment.appendChild(childNodes[j].cloneNode(true));
				}
				
				parentNode.replaceChild(tempFragment, anchorElements[i]);
				i--; //List is live, so replacing the node means that anchorElements[i] is refering to the next node already
			}
		}
	},
	
	BlockSiteMain: function(event)
	{
		const BlockSitePrefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("BlockSite.");
		
		if(BlockSitePrefBranch.prefHasUserValue("enabled") && BlockSitePrefBranch.getBoolPref("enabled"))
		{			
			// earlier versions did not have a remove links preference, so only disable the functionality if the user has
			// specifically unchecked the "remove links" checkbox in the preferences
			if(BlockSitePrefBranch.prefHasUserValue("removeLinks") && (BlockSitePrefBranch.getBoolPref("removeLinks") == false))
			{
				return;
			}

			if(event.type === "DOMContentLoaded" || event.type == "change")
			{			
				// Anchors
				BlockSite.processAnchors(event);
			}
		}
	}
}

// Event listener
window.document.getElementById("appcontent").addEventListener("DOMContentLoaded", BlockSite.BlockSiteMain, false);

// Observer for HTTP requests to block the sites we don't want
var BlockSiteObserver = 
{
	observe: function(aSubject, aTopic, aData)
	{
		if (aTopic != 'http-on-modify-request')
			return;

		aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
		
		const BlockSitePrefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("BlockSite.");
		
		if(BlockSitePrefBranch.prefHasUserValue("enabled") && BlockSitePrefBranch.getBoolPref("enabled"))
		{
			if (BlockSite.checkLocation(aSubject.URI.spec))
			{
				if (BlockSitePrefBranch.prefHasUserValue("showWarning") && BlockSitePrefBranch.getBoolPref("showWarning"))
					BlockSite.showBlockWarningBar();
				aSubject.loadFlags = Components.interfaces.nsICachingChannel.LOAD_ONLY_FROM_CACHE;
				aSubject.cancel(Components.results.NS_ERROR_FAILURE);
			}
		}
	},

	QueryInterface: function(iid)
	{
		if (!iid.equals(Components.interfaces.nsISupports) &&
		!iid.equals(Components.interfaces.nsIObserver))
		throw Components.results.NS_ERROR_NO_INTERFACE;

		return this;
	}
};

// Add our observer
var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
observerService.addObserver(BlockSiteObserver, "http-on-modify-request", false);

// Remove observer when current window closes
window.addEventListener("unload", function() {
  observerService.removeObserver(BlockSiteObserver, "http-on-modify-request");
}, false);
